# -*- coding: UTF-8 -*-

import json


class CSConvertersConfig(object):
    """ Класс для получения настроек конвертеров для кассового сервера """
    def __init__(self, saleExchangeConfig={}):
        self.saleExchangeConfig = saleExchangeConfig

    def getInt(self, name, defaultValue):
        try:
            return int(self.getParamValue(name, defaultValue))
        except:
            return defaultValue

    def getBool(self, name, defaultValue):
        return str(self.getParamValue(name, defaultValue)).lower() in ("yes", "true", "t", "1")

    def getBoolWithoutDefaultValue(self, name):
        defaultValue = False
        return self.getBool(name, defaultValue)

    def getString(self, name, defaultValue):
        value = self.getParamValue(name, defaultValue)
        unicodeValue = value.decode('utf8') if isinstance(value, str) else value
        return unicodeValue.encode('utf8')

    def getParamValue(self, key, defaultValue=None):
        # ищем названия настроек без учета регистра
        l = filter(lambda param: param.get('key', '').lower() == key.lower(), self.saleExchangeConfig.get('params', []))
        value = l[0].get('value', defaultValue) if len(l) > 0 else defaultValue
        return value

    def getFormat(self, defaultValue):
        return self.getString('format', defaultValue)

    def getCharset(self, converterName):
        defaultValue = 'cp1251'
        if converterName in ['ns2000', 'NS2000', 'pilot', 'PILOT', 'pilot-original', 'PILOT-ORIGINAL']:
            defaultValue = 'cp866'
        return self.getString('charset', defaultValue)

    def getGroup2CatalogMapper(self):
        return json.loads(self.getParamValue('group2catalog', '{}'))

    def getValutMapper(self):
        return json.loads(self.getParamValue('valut', '{}'))

    def getMeasureMapper(self):
        return json.loads(self.getParamValue('measure', '{}'))

    def getRolesMapper(self):
        return json.loads(self.getParamValue('roles', '{}'))

    def getPaymentMapper(self):
        return json.loads(self.getParamValue('payment', '{}'))

    def getOracleMeasureMappers(self):
        return json.loads(self.getParamValue('measure', '{}'))
