# -*- coding: UTF-8 -*-

import codecs
import os
from common import filesmodify, logger, macros
from datetime import datetime

currentConverter = None
bcodeMode = {'manual': 1, 'scanner': 2, 'hotkey': 3, 'selected': 4, 'quickCode': 5, 'complect': 6, 'external': 7, 'mcr': 8, 'softCheck': 9}
saleFiles = set()


class StopSearch(Exception):
    pass


def writeHeaderCL(file, object, user, shiftEnd):
    file.write(u"КОНТРОЛЬНАЯ ЛЕНТА\n")
    file.write(u"ДАТА=" + (shiftEnd.strftime('%d/%m/%Y') if shiftEnd else datetime.now().strftime('%d/%m/%Y')) + "\n")
    file.write(u"ВРЕМЯ=" + (shiftEnd.strftime('%H:%M') if shiftEnd else datetime.now().strftime('%H:%M')) + "\n")
    file.write(u"КАССА=" + str(object['cashCode']).zfill(3) + "\n")
    file.write(u"СЕАНС=" + str(object['cashCode']).zfill(3) + "/" + str(object['shift']).zfill(6) + "\n")
    file.write(u"КАССИР=" + user + "\n")
    file.write(u"МОРДА=42\n\n")


def writeHeaderST(file, object, shiftEnd):
    file.write(u"ОТЧЕТ ПО СТОРНИРОВАННЫМ ПОЗИЦИЯМ\n")
    file.write(u"СМЕНА: " + str(object['shift']) + "\n")
    file.write(u"ДАТА: " + (shiftEnd.strftime('%d/%m/%Y') if shiftEnd else datetime.now().strftime('%d/%m/%Y')) + "\n")
    file.write(u"ВРЕМЯ: " + (shiftEnd.strftime('%H:%M:%S') if shiftEnd else datetime.now().strftime('%H:%M:%S')) + "\n\n")
    file.write(u"   DATE      ,  TIME  , TIMESTORNO,  CHECK,BARCODE      ,NAME                               ,   QUANT,   PRICE,     SUMMA,KASSIR\n")


def writeKKMInfo(file, shift):
    kkms = {}
    if shift and shift['kkms']:
        kkms = shift['kkms'][0]
    file.write(u"ПРОДАЖИ_НАЛ={0:>16.2f}\n".format(kkms.get('sumSaleByPayment1') or 0.00))
    file.write(u"ВОЗВРАТЫ_НАЛ={0:>16.2f}\n".format(kkms.get('sumBackByPayment1') or 0.00))
    file.write(u"ПРОДАЖИ_БЕЗНАЛ={0:>16.2f}\n".format(kkms.get('sumSaleByPayment2') or 0.00))
    file.write(u"ВОЗВРАТЫ_БЕЗНАЛ={0:>16.2f}\n".format(kkms.get('sumBackByPayment2') or 0.00))
    file.write(u"ПРОДАЖИ_ЧЕКИ={0:>12.2f}\n".format(kkms.get('countSale') or 0.00))
    file.write(u"ВОЗВРАТЫ_ЧЕКИ={0:>12.2f}\n".format(kkms.get('countBack') or 0.00))
    file.write(u"ПРОДАЖИ_СУММА_ИТОГ={0:>16.2f}\n".format(kkms.get('sumProtectedEnd') or 0.00))
    file.write(u"СМЕНА_НОМЕР={0:>4d}\n".format(shift['shift'] or 0))
    file.write(u"КАССА_ЗАВ_НОМЕР={0}\n".format(kkms.get('serialNum') or ""))
    file.write(u"ВЛАДЕЛЕЦ_ИНН={0}\n".format(""))
    file.write(u"ФИСКАЛИЗАЦИЯ_ДАТА={0}\n".format(""))
    file.write(u"СМЕНА_НАЧАЛО={0}\n".format(shift['timeBeg'].strftime(" %d:%m:%Y  %H:%M:%S") or ""))
    file.write(u"ПРОДАЖИ_РЕГИСТР={0:>16.2f}\n".format(kkms.get('sumSale') or 0.00))
    file.write(u"ПРОДАЖИ_СУММАТОР={0:>16.2f}\n".format(kkms.get('sumProtectedBeg') or 0.00))
    file.write(u"ВОЗВРАТЫ_АННУЛИРОВАНО={0:>16.2f}\n".format(0.00))
    file.write(u"ВОЗВРАТЫ_АННУЛИРОВАНО_ЧЕКИ={0:>7d}\n".format(0))
    file.write(u"ВНЕСЕНИЯ={0:>16.2f}\n".format(kkms.get('sumMoneyIn') or 0.00))
    file.write(u"ИЗЪЯТИЯ={0:>16.2f}\n".format(kkms.get('sumMoneyOut') or 0.00))
    file.write(u"ВНЕСЕНИЯ_ЧЕКИ={0:>7d}\n".format(kkms.get('countMoneyIn') or 0))
    file.write(u"ИЗЪЯТИЯ_ЧЕКИ={0:>7d}\n".format(kkms.get('countMoneyOut') or 0))
    file.write(u"ПРОДАЖИ={0:>16.2f}\n".format(kkms.get('sumSale') or 0.00))
    file.write(u"ВОЗВРАТЫ={0:>16.2f}\n".format(kkms.get('sumBack') or 0.00))
    file.write(u"ПРОДАЖИ_КУПОН={0:>16.2f}\n".format(0.00))
    file.write(u"ВОЗВРАТЫ_КУПОН={0:>16.2f}\n".format(0.00))
    file.write(u"ВНЕСЕНИЯ_НАЛ={0:>16.2f}\n".format(kkms.get('sumMoneyIn') or 0.00))
    file.write(u"ВНЕСЕНИЯ_БЕЗНАЛ={0:>16.2f}\n".format(0.00))
    file.write(u"ВНЕСЕНИЯ_КУПОН={0:>16.2f}\n".format(0.00))
    file.write(u"ИЗЪЯТИЯ_НАЛ={0:>16.2f}\n".format(kkms.get('sumMoneyOut') or 0.00))
    file.write(u"ИЗЪЯТИЯ_БЕЗНАЛ={0:>16.2f}\n".format(0.00))
    file.write(u"ИЗЪЯТИЯ_КУПОН={0:>16.2f}\n".format(0.00))


def openFile(log, cash, node, workDirectory, shiftNum, previousFile=None, isClFile=True):
    global saleFiles
    output = None
    firstRound = True
    headerSize = 8
    salePathMask = os.path.join(currentConverter.path, '%(cashcode_int)03d%(shiftnum_int)05d.CL')

    if not isClFile:
        headerSize = 4
        salePathMask = os.path.join(currentConverter.path, '%(cashcode_int)03d%(shiftnum_int)05d.ST')
    currentPath = macros.applyMacros(salePathMask, log, shopcode=node.id, cashcode=cash.code, shiftnum=shiftNum)
    salePath = os.path.join(workDirectory, os.path.basename(currentPath))
    if os.path.isfile(salePath):
        log.info("file exists " + salePath)
        f = codecs.open(salePath, 'r')
        if f.readline(1):
            if previousFile == salePath:
                output = codecs.open(salePath, 'a', currentConverter.charset)
        try:
            for i in range(headerSize):
                f.next()
                firstRound = False
        except StopIteration:
            firstRound = True
            if output:
                output.close()
        f.close()
        firstRound = False
    if not output or output.closed:
        output = codecs.open(salePath, 'w', currentConverter.charset)
        firstRound = True
        log.info("file is created " + salePath)
    saleFiles.add(output.name)
    return output, firstRound


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    global currentConverter
    global saleFiles
    log = logger.getUnloadLogger("Node", node.id)
    log.info("START. Cash №" + str(cash.code))
    currentConverter = node.currentConverter
    output = None
    outputST = None
    firstRound = True       # Флаг. Указывает на то, чтобы писать или не писать заголовок отчета в файл.
    firstRoundST = True
    shifts = {}
    userNames = {}
    for sh in shiftsFunc():
        shifts[sh['shift']] = sh
        userNames[sh['shift']] = {}
        for user in sh['users']:
            userNames[sh['shift']][user['usercode']] = user['username']
        output, firstRound = openFile(log, cash, node, workDirectory, str(sh['shift']))
        outputST, firstRoundST = openFile(log, cash, node, workDirectory, str(sh['shift']), isClFile=False)
        if firstRound:
            writeHeaderCL(output, sh, userNames[sh['shift']][sh['userCode']], sh['timeEnd'])
            writeKKMInfo(output, sh)
        if firstRoundST:
            writeHeaderST(outputST, sh, sh['timeEnd'])
        output.close()
        outputST.close()
    shift = None
    firstRound = True       # Флаг. Указывает на то, чтобы писать или не писать заголовок отчета в файл.
    firstRoundST = True
    for check in salesFunc():
        if firstRound:
            output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']))
        if firstRoundST:
            outputST, firstRoundST = openFile(log, cash, node, workDirectory, str(check['shift']), isClFile=False)
        if shift != check['shift']:
            if shifts.has_key(shift):
                previousFile = output.name
                previousFileST = outputST.name
                writeKKMInfo(output, shifts[shift])
                output.close()
                outputST.close()
                output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']), previousFile)
                outputST, firstRoundST = openFile(log, cash, node, workDirectory, str(check['shift']), previousFileST, isClFile=False)
            shift = check['shift']

        shiftEnd = shifts[check['shift']]['timeEnd']
        if check['userCode'] in userNames[check['shift']]:
            user = userNames[check['shift']][check['userCode']]
        else:
            user = ''
        if firstRound:
            writeHeaderCL(output, check, user, shiftEnd)
            firstRound = False
        if firstRoundST:
            writeHeaderST(outputST, check, shiftEnd)
            firstRoundST = False

        if check['docType'] not in [1, 2, 25]:
            continue

        vcode = 99
        for moneyPos in check['moneyPositions']:
            vcode = moneyPos['valCode']
            break

        cardnum = unicode()
        try:
            for card in check['cardPositions']:
                if card['number']:
                    cardnum = card['number']
                    raise StopSearch
        except StopSearch:
            pass

        myctype = unicode('08')
        if vcode in [1, 2]:
            myctype = '06'
            if check['docSum'] != check['baseSum']:
                myctype = '07'
        if check['docType'] in [2, 25]:
            if myctype == '08':
                myctype = '04'
            else:
                myctype = '86'
        if check['closed'] == 1:
            strTrans = u'Р=%s,%s,%s,"%s","",001,%s,%s,%s,%s\n' % (str(check['docNum']).zfill(8),
                                                                 check['timeEnd'].strftime('%d/%m/%Y'),
                                                                 check['timeEnd'].strftime('%H:%M:%S'),
                                                                 myctype,
                                                                 ("%.2f" % abs(check['docSum'])).rjust(12),
                                                                 ("%.2f" % abs(check['baseSum'])).rjust(12),
                                                                 cardnum.ljust(17),
                                                                 check['userCode'].rjust(8))
            output.write(strTrans)

        for invent in check['inventPositions']:
            strTrans = u'Т=%s,%s,%s,"%s","1",1,"%s",1,1,1,%s,2,1,2,2,%s,%s,"%s",%s,"%s",%s,%s\n' % (
                str(check['docNum']).zfill(8),
                invent['posTime'].strftime('%d/%m/%Y'),
                invent['posTime'].strftime('%H:%M:%S'),
                myctype,
                invent['inventCode'].zfill(5),
                ("%.2f" % (invent['modSum'] / invent['accountingQuant'])).rjust(13),
                ("%.3f" % invent['accountingQuant']).rjust(7),
                ("%.2f" % invent['posSum']).rjust(10),
                str(invent['barCode']).ljust(13),
                bcodeMode.get(invent['bcodeMode'], 0),
                invent['excisemark'] if invent['excisemark'] else "",
                ("%.2f" % invent['vatrate1']).rjust(6),
                ("%.2f" % invent['vatsum1']).rjust(14))
            output.write(strTrans)

        for stornoinvent in check['stornoPositions']:
            strTrans = u'ST=%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n' % (
                stornoinvent['posTime'].strftime('%d/%m/%Y'),
                stornoinvent['posTime'].strftime('%H:%M:%S'),
                stornoinvent['timeStorno'].strftime('%H:%M:%S'),
                str(check['docNum']).zfill(8),
                stornoinvent['barCode'].ljust(13),
                stornoinvent['name'][:32].ljust(32),
                ("%.3f" % stornoinvent['accountingQuant']).rjust(8),
                ("%.2f" % (stornoinvent['modSum'] / stornoinvent['accountingQuant'])).rjust(8),
                ("%.2f" % stornoinvent['posSum']).rjust(10),
                stornoinvent['userCode'].rjust(6))
            outputST.write(strTrans)

    if output:
        if shift:
            writeKKMInfo(output, shifts[shift])
        output.close()
        converterSalesReadyFile = open('%s/ready' % (workDirectory), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
        salesReadyFile.close()
        saleFiles.add(converterSalesReadyFile.name)
        saleFiles.add(salesReadyFile.name)

    if outputST:
        outputST.close()

    if shift:
        shifts[shift] = ''
    log.info("Unloaded the following number of shifts: %s" % shifts.keys())
    log.info("FINISH")


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, node.currentConverter.path, backupDir=backupDir,
            rewrite=True, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    return False, []


def getSaleFiles():
    global saleFiles
    return saleFiles
