# -*- coding: UTF-8 -*-

import ast
from sqlUnloader import SQLUnloader, CaseInsensitiveDict


class SecurityLogUnloader(SQLUnloader):
    def __init__(self, connection, log):
        self.cursor = connection.cursor()
        self.sdata = connection.cursor()
        self.log = log
    
    def getExtrainfo(self, commandname):
        try:
            return ast.literal_eval(commandname)
        except:
            pass
        return {}

    def unloadSecurityLogObjects(self):
        for row in self.sdata:
            dict_lower = CaseInsensitiveDict(row)
            yield {'crightsid': self.getValue(dict_lower, 'crights.crightsid'),
                    'shiftnum': self.getValue(dict_lower, 'workshift.shiftnum'),
                    'cashiercard': self.getValue(dict_lower, 'crights.cashiercard'),
                    'cashiername': self.getValue(dict_lower, 'crights.cashiername'),
                    'actorcard': self.getValue(dict_lower, 'crights.actorcard'),
                    'actorname': self.getValue(dict_lower, 'crights.actorname'),
                    'op': self.getValue(dict_lower, 'crights.op'),
                    'isgoodsitem': self.getValue(dict_lower, 'crights.isgoodsitem'),
                    'optime': self.getValue(dict_lower, 'crights.optime'),
                    'opcode': self.getValue(dict_lower, 'crights.opcode'),
                    'commandname': self.getValue(dict_lower, 'crights.commandname'),
                    'cashcode': self.getValue(dict_lower, 'crights.cashcode'),
                    'shifttype': self.getValue(dict_lower, 'workshift.shifttype') if self.getValue(dict_lower, 'workshift.shifttype') else 0,
                    'extrainfo': self.getExtrainfo(self.getValue(dict_lower, 'crights.commandname')),
                    'identifier': self.getValue(dict_lower, 'crights.identifier')
                   }

    def unloadSecurityLogs(self, shiftIds):
        if shiftIds:
            self.log.info('unload securityLogs for shiftIds %s' % (shiftIds,))
            query = "select `crights`.*, `workshift`.`shiftnum`, `workshift`.`shifttype` from `crights`, `workshift` where \
                `crights`.`workshiftid` IN %s and `crights`.`workshiftid`=`workshift`.`workshiftid` ORDER BY `crightsid`"
            self.sdata.execute(query, (shiftIds,))
            for o in self.unloadSecurityLogObjects():
                yield o
        else:
            self.log.info(u'Журнал выполненных операций пуст')

    def unloadSecurityLogsByDate(self, shiftIds, fromDate, toDate):
        if shiftIds:
            self.log.info('unload securityLogs from %s to %s' % (str(fromDate), str(toDate)))
            self.sdata.execute("select `crights`.*, `workshift`.`shiftnum`, `workshift`.`shifttype` from `crights`, `workshift` where \
                `crights`.`workshiftid` IN %s and `crights`.`workshiftid`=`workshift`.`workshiftid` and (%s IS NULL or `crights`.`optime` > %s) \
                and (%s IS NULL or `crights`.`optime` <= %s) ORDER BY `crightsid`", (shiftIds, fromDate, fromDate, toDate, toDate,))
            for o in self.unloadSecurityLogObjects():
                yield o

    def unloadSecurityLogsByDocuments(self, docIds):
        if docIds:
            self.log.info('unload securityLogs for docIds %s' % (docIds,))

            self.cursor.execute('SELECT DISTINCT(`workshiftid`) FROM `document` WHERE `documentid` IN %s', (docIds,))
            shiftIds = []
            for shiftId in self.cursor.fetchall():
                self.log.info('shiftId = %s' % shiftId)
                shiftIds.append(int(shiftId.get('workshiftid')))
            shiftIds = '('+str(shiftIds)[1:-1]+')'
            self.cursor.execute('SELECT MIN(`time_beg`) AS `min_time_beg` FROM `document` WHERE `documentid` IN %s', (docIds,))
            fromDate = self.cursor.fetchone().get('min_time_beg')
            self.cursor.execute('SELECT MAX(`time_end`) AS `max_time_beg` FROM `document` WHERE `documentid` IN %s', (docIds,))
            toDate = self.cursor.fetchone().get('max_time_beg')

            return self.unloadSecurityLogsByDate(shiftIds, fromDate, toDate)
        else:
            self.log.info('Documents not found')
            return ()
