# -*- coding: UTF-8 -*-

import os
import stat
from common import filesmodify, logger, macros
from common import statuses
from common.rest_state import RestState as rest_state
from common.statuses import unload as st

from unload.nes.unload import Unloader
from datetime import datetime


class Reader(object):
    def __init__(self):
        self.createSalesFile = False
        self.convertedSaleStateIds = {}
        self.cashSaleStateIds = {}
        self.finishFlagPath = None
        self.processFlagPath = None

    def read(self, node, closedShifts, shifts, dates, granularity, changedShifts, workDirectory, converterName=None,
            cashCode=None, cashUnloadStateId=None, statusHost=None, isSendDebugStatus=True, unloadToDay=None):
        log = logger.getUnloadLogger("Node", node.id)
        log.info("Reading...")
        log.info("Node id = %s" % node.id)

        unloader = Unloader(node, log)

        granularityDate = True  # детализация по датам документов

        if shifts is not None:
            (fromShift, toShift) = shifts.split('@')
            (fromShift, toShift) = (int(fromShift), int(toShift))

        if dates is not None:
            (fromDate, toDate) = dates.split('@')
            (fromDate, toDate) = (datetime.strptime(fromDate, '%Y-%m-%dT%H:%M:%S.%f'), datetime.strptime(toDate, '%Y-%m-%dT%H:%M:%S.%f'))
            if granularity == 'shift':
                granularityDate = False

        self.cashSaleStateIds[node.id] = {}
        cashList = []
        for cash in node.cashes:
            if cashCode and cash.code != cashCode:
                continue

            if not converterName or converterName == "oracle":
                cashHasOracleConverter = False
                for converter in cash.converters:
                    if converter.name == "oracle":
                        oracleConverter = converter
                        cashList.append(cash.code.encode('utf-8'))
                        cashHasOracleConverter = True
                        break
                if (cashHasOracleConverter and len(cash.converters) == 1) or converterName == "oracle":
                    continue
            statuses.setUnloadStatus(st.GET_DOCUMENTS)
            if dates is not None:
                if granularityDate:
                    (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadByDocumentDates(cash.code, fromDate, toDate)
                else:
                    (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadByShiftDates(cash.code, fromDate, toDate, closedShifts)
            elif shifts is not None:
                if changedShifts:
                    (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadByShiftsAndSetChanged(cash.code, fromShift, toShift, closedShifts)
                else:
                    (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadByShifts(cash.code, fromShift, toShift, closedShifts)
            elif changedShifts:
                (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadChangedShifts(cash.code, closedShifts)
            elif unloadToDay:
                (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadOpenedShiftsToDay(cash.code)
            else:
                (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadChanged(cash.code, closedShifts)

            self.convertedSaleStateIds = {}
            converterIndex = {}
            for converter in cash.converters:
                if converter.name == "oracle" or (converterName and converter.name != converterName):
                    continue
                indexedConverterName = self.getIndexedName(converter.name, converterIndex)
                unloadDirectory = self.getUnloadDirectory(workDirectory, node.id, indexedConverterName, closedShifts == 1, cash.code)

                log.debug('shopCode = %s, cashCode = %s, converter = %s' % (node.id, cash.code, converter.name))
                if not self.runUnload(node, cash, cash.code, converter, closedShifts, unloadDirectory, converterName, cashUnloadStateId, statusHost, isSendDebugStatus, unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload):
                    return False
            if setFlagUnload is not None and self.createSalesFile:
                statuses.setUnloadStatus(st.SET_FLAG_UNLOAD)
                log.debug('SET FLAG UNLOAD')
                setFlagUnload()
            self.cashSaleStateIds[node.id][cash.code] = self.convertedSaleStateIds
        if cashList:
            if dates:
                (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadByShop(fromDate, toDate, cashList)
            elif shifts:
                (unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload) = unloader.unloadByShiftsForShop(fromShift, toShift, cashList, closedShifts)
            unloadDirectory = self.getUnloadDirectory(workDirectory, node.id, converter.name, closedShifts == 1, cashList[0])
            log.debug('shopCode = %s, converter = %s' % (node.id,  converter.name))
            if not self.runUnload(node, None, node.id, oracleConverter, closedShifts, unloadDirectory, converterName , cashUnloadStateId, statusHost, isSendDebugStatus, unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload):
                return False
            self.cashSaleStateIds[node.id][cashList[0]] = self.convertedSaleStateIds
        return True

    def send(self, node, workDirectory, cashUnloadStateId=None, statusHost=None, isSendDebugStatus=True, closedShifts=0):
        log = logger.getUnloadLogger("Node", node.id)
        log.info("Sending...")
        currentSales = closedShifts == 1
        result = True
        for cash in node.cashes:
            converterIndex = {}
            for converter in cash.converters:
                log.info('work with converter:%s' % converter.name)
                indexedConverterName = self.getIndexedName(converter.name, converterIndex)
                readyFile = self.getConverterStatusFileName(workDirectory, node.id, cash.code, indexedConverterName, currentSales)
                log.info('readyFile=%s' % readyFile)
                if os.path.exists(readyFile):
                    log.info('Send sales from cashCode=%s, converter=%s' % (cash.code, converter.name))
                    node.currentConverter = converter
                    backupDir = converter.backup_dir if converter.backup_dir else cash.backup_dir
                    unloadDirectory = self.getUnloadDirectory(workDirectory, node.id, indexedConverterName, currentSales, cashCode=cash.code)
                    if os.path.exists(unloadDirectory):
                        try:
                            allFileProcessed = False
                            plugin = __import__('unload.converter.%s.%s' % (converter.name, converter.name), fromlist='q')
                            processFlagPath = self.createFileFlag(log, self.processFlagPath)
                            if currentSales:
                                allFileProcessed, files = plugin.sendOnlineSales(cash, node, unloadDirectory)
                            else:
                                allFileProcessed, files = plugin.sendSales(cash, node, unloadDirectory, backupDir)

                            if not allFileProcessed:
                                result = False
                                state = {'messageEn': 'Not all files processed', 'messageRu': u'Не все файлы обработаны', 'status': rest_state.STATUS_ERROR, 'resultFiles': files}
                                rest_state.updateSaleState(node, self.cashSaleStateIds.get(node.id, {}).get(cash.code, {}).get(converter.name, []),
                                        state, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
                            else:
                                os.remove(readyFile)
                                statuses.setUnloadStatus(st.SALES_SENT)
                                state = {'messageEn': 'Sales was send', 'messageRu': u'Продажи успешно выгружены', 'status': rest_state.STATUS_READY, 'resultFiles': files}
                                rest_state.updateSaleState(node, self.cashSaleStateIds.get(node.id, {}).get(cash.code, {}).get(converter.name, []),
                                        state, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
                                if processFlagPath and os.path.exists(processFlagPath):
                                    os.remove(processFlagPath)
                                self.createFileFlag(log, self.finishFlagPath)
                        except Exception as e:
                            statuses.setUnloadStatus(st.ERROR_SEND_SALES)
                            state = {'messageEn': 'An error occurred during sending sales: %s' % str(e), 'messageRu': u'Произошла ошибка на заключительном этапе выгрузки продаж', 'status': rest_state.STATUS_ERROR}
                            rest_state.updateSaleState(node, self.cashSaleStateIds.get(node.id, {}).get(cash.code, {}).get(converter.name, []),
                                    state, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
                            log.error(e)
                            return False
        log.info('result of send sale:%s' % result)
        if result:
            os.remove(self.getNodeStatusFileName(workDirectory, node.id, currentSales))
        else:
            statuses.setUnloadStatus(st.ERROR_SEND_SALES)
        return result

    def runUnload(self, node, cash, cashCode, converter, closedShifts, unloadDirectory, converterName , cashUnloadStateId, statusHost, isSendDebugStatus, unloadShifts, unloadDocuments, unloadSecurityLogs, setFlagUnload):
        log = logger.getUnloadLogger("Node", node.id)
        currentSales = closedShifts == 1
        if currentSales and sum(1 for x in unloadShifts()) == 0:
            return True
        shiftNums = []
        for shiftEntity in unloadShifts():
            shiftNums.append(shiftEntity['shift'])
        saleStateIds = rest_state.sendSaleState(node, cashCode, shiftNums, 'Start unload sales', u'Начинаем выгружать продажи',
                rest_state.STATUS_DURING, converter.name, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
        node.currentConverter = converter
        if not os.path.exists(unloadDirectory):
            os.makedirs(unloadDirectory)
        try:
            statuses.setUnloadStatus(st.CONVERT_START)
            plugin = __import__('unload.converter.%s.%s' % (converter.name, converter.name), fromlist='q')
            plugin.run(unloadDocuments, unloadSecurityLogs, unloadShifts, cash, node, unloadDirectory, currentSales)
            self.setUnloadFilesPermissions(log, plugin)
            statuses.setUnloadStatus(st.CONVERT_END)
            state = {'messageEn': 'Unload and converting sales performed', 'messageRu': u'Выгрузка и конвертирование продаж выполнены', 'status': rest_state.STATUS_DURING}
            rest_state.updateSaleState(node, saleStateIds, state, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
            self.createSalesFile = True
        except Exception, e:
            statuses.setUnloadStatus(st.ERROR_CONVERT)
            state = {'messageEn': 'An error occurred during unloading and converting sales: %s' % str(e), 'messageRu': u'Произошла ошибка во время выгрузки и конвертирования продаж', 'status': rest_state.STATUS_ERROR}
            rest_state.updateSaleState(node, saleStateIds, state, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
            log.error(e)
            return False
        self.convertedSaleStateIds[converter.name] = saleStateIds
        return True

    def setUnloadFilesPermissions(self, log, plugin):
        try:
            for fileName in plugin.getSaleFiles():
                filesmodify.addFilePermissions(log, fileName, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)
        except Exception, e:
            log.warn(repr(e))

    def getUnloadDirectory(self, workDirectory, nodeId, converterName, currentSales, cashCode=None):
        unloadDir = ""
        if cashCode:
            unloadDir = workDirectory + '/%s/unload/%s/%s' % (nodeId, cashCode, converterName)
        else:
            unloadDir = workDirectory + '/%s/unload/%s' % (nodeId, converterName)

        if currentSales:
            unloadDir += '/online'

        return unloadDir

    def getConverterStatusFileName(self, workDirectory, nodeId, cashCode, converterName, currentSales):
        name = workDirectory +'/%s/unload/%s/%s/ready' % (nodeId, cashCode, converterName)
        if currentSales:
            name += "Online"
        return name

    def getNodeStatusFileName(self, workDirectory, nodeId, currentSales):
        name = workDirectory + '/%s/unload/ready' % (nodeId)
        if currentSales:
            name += "Online"
        return name

    def deleteCurrentSales(self, node, workDirectory):
        log = logger.getUnloadLogger("Node", node.id)
        log.info(u"Удаление файлов текщих продаж")
        for cash in node.cashes:
            converterIndex = {}
            for converter in cash.converters:
                node.currentConverter = converter
                indexedConverterName = self.getIndexedName(converter.name, converterIndex)
                plugin = __import__('unload.converter.%s.%s' % (converter.name, converter.name), fromlist='q')
                try:
                    plugin.deleteCurrentSales(node, self.getUnloadDirectory(workDirectory, node.id, indexedConverterName, True, cashCode=cash.code))
                except AttributeError, e:
                    log.warn(u"Конвертер '%s' не поддерживает удаление текущих продаж: %s" % (converter.name, e.message))

        log.info(u"Удаление завершено")

    def getIndexedName(self, converter, converterIndex):
        """ Возвращает рабочую директорию с добавлением индекса(номер повторяющегося конвертера), когда настроено несколько одинаковых конвертеров.
        Иначе не возможно выгружать данные в несколько директорий, так как копирование из рабочей директории было сразу всех данных (нескольких конвертеров) """
        index = 0
        if converter in converterIndex:
            index = converterIndex[converter]
            converterIndex[converter] = index + 1
        else:
            converterIndex[converter] = 1

        return (converter + ('%s' % index)) if index > 0 else converter

    def createFileFlag(self, log, fileFlagPath):
        """
        Создает файл-флаг, соответствующий настройкам в exchangers.ini
        flagType - строка, указывающая тип создоваемого файла-флага. Возможные значения:
            - fileFlagPath - путь до файла-флага
        """
        if fileFlagPath:
            fileFlagPath = macros.applyMacros(fileFlagPath, log)
            log.info(u"Создаём файл-флаг: %s", fileFlagPath)
            directory = os.path.dirname(fileFlagPath)
            if not os.path.exists(directory):
                os.makedirs(directory)
            unloadFinishFlag = open(fileFlagPath, "w")
            unloadFinishFlag.close()
            filesmodify.addFilePermissions(log, unloadFinishFlag.name, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)
        return fileFlagPath
