# -*- coding: UTF-8 -*-

import os
import sys
import fnmatch
import time
from artix.common import servicelock
from artix.common.progress import Progress
from common import config, logger, macros
from common import filesmodify
from common.rest_state import RestState as rest_state
from common import statuses
from common.basictaskrunner import BasicTaskRunner
from common.statuses import upload as st 
from upload.inserter import InserterError
import atexit
import tempfile
import traceback
import shutil
import datetime

log = logger.getUploadLogger("main")
lockFile = os.path.normpath('%s/locks/lockArtixFAM' % tempfile.gettempdir())
taskRunner = None
maxCountBackupFiles = None
actualCountDaysForBackupFiles = None
stopConvertIfBadDict = True
progress = Progress(log)
testProgressMode = False
errorStatDir = "/linuxcash/logs/defects/exchangers"

srvLock = servicelock.ServiceLock(lockFile, log)
atexit.register(srvLock.unlock)


def return_result_convert(result, node, workDirectory):
    progress.removeProgresses(testProgressMode)
    log = logger.getUploadLogger("Node", node.id)
    if not result and not stopConvertIfBadDict:
        destDir = os.path.join(node.badDictsDirectory, node.id, node.currentConverter.name, str(int(time.time()*100)))
        log.info(u'Переносим данные, при конвертировании которых произошла ошибка. %s --> %s' % (workDirectory, destDir))
        os.makedirs(destDir)
        filesmodify.safeMoveDirectory(log, workDirectory, destDir, rewrite=True)
        result = True
    return result


def getBackupDir(node):
    return node.currentConverter.backup_dir if node.currentConverter.backup_dir else node.backup_dir


def rebuild_convert(workDir, node):
    result = False
    log = logger.getUploadLogger("Node", node.id)
    try:
        removeError(node)
        plugin = __import__('upload.converter.%s' % (node.currentConverter.name), fromlist='q')
        log.info(u'Перезапуск конвертации - %s для %s' % (node.currentConverter.name, node.id))
        statuses.setUploadStatus(st.START_REBUILD_CONVERT)
        #result = plugin.rebuild_convert(workDir, node)
        data = filesmodify.getFilesByMask(workDir, node.currentConverter.data)
        if len(data) < 1:
            log.info(u'Файлы данных не найдены')
            flags = [os.path.join(workDir, os.path.basename(flag)) for flag in node.currentConverter.flags]
            filesmodify.removeFiles(flags)
            return return_result_convert(True, node, workDir)
        if node.currentConverter.name.startswith('smag') or node.currentConverter.name == 'cif1':
            data = [data]
        progress.createProgress(None, 0)
        data.sort()
        for i in range(len(data)):
            node.currentConverter.dict_state_id = rest_state.getDictState(node, data[i])
            node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Restart convert dict', u'Перезапуск конвертации справочника', rest_state.STATUS_DURING, data[i])
            result, errorList = plugin.general_method(workDir, data[i] if isinstance(data[i], list) else [data[i]], node)
            errorObjectsCount = plugin.invalidObjectsCount()
            for error in errorList:
                saveError(error, node)
            if result:
                if errorObjectsCount > 0:
                    log.info(u'Справочник загружен, но не полностью. %s объектов с ошибками' % errorObjectsCount)
                    node.currentConverter.dict_state_id = rest_state.sendDictState(node,
                            u'Dictionary is not fully loaded. %s invalid objects' % errorObjectsCount,
                            u'Справочник загружен, но не полностью. %s объектов с ошибками' % errorObjectsCount,
                            rest_state.STATUS_READY_WITH_ERROR, data[i])
                else:
                    node.currentConverter.dict_state_id = rest_state.sendDictState(node, u'OK', u'ОК', rest_state.STATUS_READY, data[i])
                createFileFlag(errorObjectsCount)

                if isinstance(data[i], list) or (i+1) == len(data): # если в data[i] не имя файла, а список файлов или если обрабатываем последний справочник, то делаем бэкап всей директории
                    fileName = None
                else:
                    fileName = os.path.basename(data[i])
                filesmodify.archive(log, workDir, getBackupDir(node), maxCountFiles=maxCountBackupFiles, actualCountDays=actualCountDaysForBackupFiles, fileName=fileName)
            else:
                saveError(u'Ошибка конвертирования', node)
                node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Error', u'Ошибка конвертирования', rest_state.STATUS_ERROR, data[i])
                return return_result_convert(result, node, workDir)
        return return_result_convert(result, node, workDir)
    except InserterError as e:
        raise e
    except Exception as e:
        log.error(u'Ошибка: %s' % str(e))
        log.error(traceback.format_exc())
        statuses.setUploadStatus(st.ERROR_CONVERT)
        saveError(e.message, node)
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, e.message, u'Ошибка: %s' % e.message, rest_state.STATUS_ERROR, None)
        return return_result_convert(False, node, workDir)


def convert(workDirectory, node):
    result = False
    log = logger.getUploadLogger("Node", node.id)
    try:
        removeError(node)
        plugin = __import__('upload.converter.%s' % (node.currentConverter.name), fromlist='q')
        log.info(u'Старт конвертирования - %s для %s' % (node.currentConverter.name, node.id))
        statuses.setUploadStatus(st.START_CONVERT)
        #result = plugin.convert(workDirectory, node)
        workDirectory = os.path.join(workDirectory, node.id, node.currentConverter.name)
        if not os.path.exists(workDirectory):
            os.makedirs(workDirectory)
        log.info(u'Создаем флаг признака копирования файлов %s' % (workDirectory + u'/copy.flag'))
        open(workDirectory + '/copy.flag', 'a').close()
        checkMatchFileNames = hasattr(node.currentConverter, 'checkMatchFileNames') and node.currentConverter.checkMatchFileNames
        flags = []
        if checkMatchFileNames:
            flags = filesmodify.getFilesByMask(os.path.dirname(node.currentConverter.flags[0]), node.currentConverter.flags)
            data = filesmodify.copyFilesByFlags(flags, workDirectory)
        else:
            data = filesmodify.copyFiles(node.currentConverter.data, workDirectory)
        if len(data) < 1:
            log.info(u'Файлы данных по %s не найдены' % (u"именам флагов " + str(flags) if checkMatchFileNames
                                                         else u"маске " + str(node.currentConverter.data)))
            filesmodify.removeFiles(node.currentConverter.flags)
            shutil.rmtree(workDirectory)
            return return_result_convert(True, node, workDirectory)
        filesmodify.moveFiles(node.currentConverter.flags, workDirectory)
        if node.currentConverter.name not in ['users', 'smag-ukm']:
            if checkMatchFileNames:
                filesmodify.removeFilesByFlags(flags)
            else:
                filesmodify.removeFiles(node.currentConverter.data)
        log.info(u'Удаляем флаг признака копирования файлов')
        os.remove(workDirectory + '/copy.flag')
        if node.currentConverter.name.startswith('smag') or node.currentConverter.name == 'cif1':
            data = [data]
        progress.createProgress(None, 0)
        data.sort()
        for i in range(len(data)):
            log.info(u'Конвертирование справочника: ')
            log.info(data[i])
            node.currentConverter.dict_state_id = None
            node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Start convert dict', u'Начало конвертации справочника', rest_state.STATUS_DURING, data[i])
            result, errorList = plugin.general_method(workDirectory, data[i] if isinstance(data[i], list) else [data[i]], node)
            errorObjectsCount = plugin.invalidObjectsCount()
            for error in errorList:
                saveError(error, node)
            if result:
                if errorObjectsCount > 0:
                    log.info(u'Справочник загружен, но не полностью. %s объектов с ошибками' % errorObjectsCount)
                    node.currentConverter.dict_state_id = rest_state.sendDictState(node,
                            u'Dictionary is not fully loaded. %s invalid objects' % errorObjectsCount,
                            u'Справочник загружен, но не полностью. %s объектов с ошибками' % errorObjectsCount,
                            rest_state.STATUS_READY_WITH_ERROR, data[i])
                else:
                    log.info(u'Конвертирование успешно завершено')
                    node.currentConverter.dict_state_id = rest_state.sendDictState(node, u'OK', u'ОК', rest_state.STATUS_READY, data[i])
                createFileFlag(errorObjectsCount)

                if isinstance(data[i], list) or (i+1) == len(data): # если в data[i] не имя файла, а список файлов или если обрабатываем последний справочник, то делаем бэкап всей директории
                    fileName = None
                else:
                    fileName = os.path.basename(data[i])
                filesmodify.archive(log, workDirectory, getBackupDir(node), maxCountFiles=maxCountBackupFiles, actualCountDays=actualCountDaysForBackupFiles, fileName=fileName)
            else:
                log.error(u'Конвертирование завершено с ошибкой')
                saveError(u'Ошибка конвертирования', node)
                node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Error', u'Ошибка конвертирования', rest_state.STATUS_ERROR, data[i])
                return return_result_convert(result, node, workDirectory)
        return return_result_convert(result, node, workDirectory)
    except InserterError as e:
        raise e
    except Exception as e:
        log.error(u'Ошибка: %s' % str(e))
        log.error(traceback.format_exc())
        statuses.setUploadStatus(st.ERROR_CONVERT)
        saveError(str(e), node)
        rest_state.sendDictState(node, str(e), u'Ошибка: %s' % str(e), rest_state.STATUS_ERROR)
        return return_result_convert(False, node, workDirectory)


def findFlag(directory, pattern):
    for file in os.listdir(directory):
        if fnmatch.fnmatch(file.lower(), pattern.lower()):
            statuses.setUploadStatus(st.FOUND_FLAG)
            return file
    return None


def start(service=True, tRunner=BasicTaskRunner()):
    # Режим service = False для тестов
    # Из тестов напрямую вызывается этот метод
    
    global taskRunner
    taskRunner = tRunner
    global maxCountBackupFiles
    global actualCountDaysForBackupFiles
    global stopConvertIfBadDict
    global testProgressMode
    global defaultStatusServer
    global defaultNesServer

    if service:
        srvLock.lock()

    testProgressMode = config.getBool('main', 'testProgressMode', False)

    workDirectory = config.getString('main', 'workDirectory', '/linuxcash/cash/data/tmp/exchangers')
    if not os.path.exists(workDirectory):
        os.makedirs(workDirectory)
    timeoutNodeReread = config.getInt('UPLOAD', 'timeoutNodeReread', 300)
    timeoutFlagFind = config.getInt('UPLOAD', 'timeoutFlagFind', 20)
    maxCountBackupFiles = config.getInt('UPLOAD', 'maxCountBackupFiles', 30)
    actualCountDaysForBackupFiles = config.getInt('UPLOAD', 'actualCountDaysForBackupFiles', 30)
    stopConvertIfBadDict = config.getBool('UPLOAD', 'stopConvertIfBadDict', True)
    defaultStatusServer = config.getString('CS', 'defaultStatusServer', 'localhost:8080')
    defaultNesServer = config.getString('CS', 'defaultNesServer', 'localhost:28081')
    taskRunner.init(config.getInt('UPLOAD', 'poolProcessSize', 1))
    if not service:
        timeoutFlagFind = 0
    
    tasks = {}
    queue = []
    nodes = []
    try:
        nodeGetter = __import__('upload.node.%s.nodeGetter' % (config.getString('UPLOAD', 'nodeGetter', 'cash')), fromlist='q')
        # внести глобальныую переменную в другой модуль (если что, она не очень глобальная, https://stackoverflow.com/questions/15959534/python-visibility-of-global-variables-in-imported-modules)
        nodeGetter.defaultNesServer = defaultNesServer
        nodeGetter.defaultStatusServer = defaultStatusServer
        nodes = nodeGetter.getNodes()
        log.debug(u'ноды = %s' % ' '.join(str(e) for e in nodes))
    except Exception as e:
        statuses.setUploadStatus(st.ERROR_READ_NODES)
        saveError(u'Ошибка считывания нод')
        log.error(traceback.format_exc())
        log.error(u'Ошибка считывания нод. %s' % str(e))

    forever = True
    while forever:
        forever = service
        try:
            log.info(u'Проверяем рабочую директорию на присутствие незагруженных справочников')
            for localDirName in os.listdir(workDirectory):  # По списку файлов в рабочей директории
                if os.path.isdir(os.path.join(workDirectory, localDirName)):  # Если файл - директория
                    nodeDir = os.path.join(workDirectory, localDirName)
                    for localDirNameConverter in os.listdir(nodeDir):
                        if os.path.isdir(os.path.join(nodeDir, localDirNameConverter)):  # Если файл - директория
                            nodeConverterDir = os.path.join(nodeDir, localDirNameConverter)
                            for node in nodes:  # По всем полученным узлам
                                if node.id == localDirName:  # Если есть директория с названием как идентификатор узла
                                    for file in os.listdir(nodeConverterDir):
                                        if fnmatch.fnmatch(file.lower(), 'copy.flag'): 
                                            log.info(u'Найден флаг признака копирования файлов %s' % (nodeConverterDir + '/copy.flag'))
                                            for f in os.listdir(nodeConverterDir):
                                                os.remove(nodeConverterDir + '/' + f)
                                            log.info(u'Каталог %s очищен' % nodeConverterDir)
                                            break
                                    node.currentConverter = None
                                    for conv in node.converters:
                                        if conv.name == localDirNameConverter:
                                            node.currentConverter = conv
                                            break
                                    if node.currentConverter:
                                        for fl in node.currentConverter.flags:
                                            node.currentFlag = findFlag(nodeConverterDir, os.path.basename(fl))
                                            if node.currentFlag:
                                                break
                                        if node.id not in tasks or tasks[node.id].ready():
                                            tasks[node.id] = taskRunner.addAsyncTask(rebuild_convert, [nodeConverterDir, node])
                                            log.info(u'Добавление задачи - %s' % nodeConverterDir)
                                            break

            timeNewNode = 0
            log.info(u'Начинаем мониторить директории для каждого узла на появление файлов-флагов загрузки справочников')
            
            forever = True
            while forever:
                forever = service
                progress.removeProgresses(testProgressMode)
                if timeNewNode >= timeoutNodeReread:
                    timeNewNode = 0
                    try:
                        mewNodes = nodeGetter.getNodes()
                        nodes = mewNodes
                    except:
                        pass
            
                for n in nodes:
                    if (n.id not in tasks) or (tasks[n.id].ready() and tasks[n.id].get()):
                        for converter in n.converters:
                            for fl in converter.flags:
                                if os.path.exists(os.path.dirname(fl)):
                                    curFlag = findFlag(os.path.dirname(fl), os.path.basename(fl))
                                    if curFlag:
                                        n.currentFlag = curFlag
                                        n.currentConverter = converter
                                        queue.append(n)
                                        break
                    elif tasks[n.id].ready() and not tasks[n.id].get():
                        deleteFromTasks = False
                        for converter in n.converters:
                            nodeWorkDir = os.path.join(workDirectory, n.id, converter.name)
                            if not os.path.exists(nodeWorkDir) or len(os.listdir(nodeWorkDir)) == 0:
                                deleteFromTasks = True
                        if deleteFromTasks and n.id in tasks:
                            log.info(u'Возобновляем работу для ноды ' + n.id)
                            del tasks[n.id]

                countPass = 5
                for i in range(countPass):
                    countDel = 0
                    for k in range(len(queue)):
                        node = queue[k - countDel]
                        id = node.id
                        if (id in tasks and tasks[id].ready() and tasks[id].get()) or (id not in tasks):
                            log.info(u'Добавление ноды - ' + id)
                            tasks[id] = taskRunner.addAsyncTask(convert, [workDirectory, node])
                            queue.remove(node)
                            countDel += 1
                    time.sleep(timeoutFlagFind / countPass)
                    timeNewNode = timeNewNode + timeoutFlagFind / countPass
        except Exception as e:
            log.error(u'Ошибка: %s' % e)
            log.error(traceback.format_exc())
            statuses.setUploadStatus(st.ERROR_CONVERT)
            time.sleep(timeoutFlagFind)


def stop():
    log.info(u'Остановка сервиса...')
    if taskRunner:
        taskRunner.stopAllTasks()
    statuses.setUploadStatus(st.PROGRAM_FINISH)
    sys.exit()


def removeError(node):
    try:
        if node.currentConverter.errorDest and os.path.exists(node.currentConverter.errorDest):
            os.remove(node.currentConverter.errorDest)
    except Exception as e:
        log.error(u'Ошибка удаления %s. %s' % (node.currentConverter.errorDest, str(e)))


def saveError(message, node=None):
    try:
        if node and node.currentConverter.errorDest:
            with open(node.currentConverter.errorDest, 'a') as outfile:
                outfile.write(message.encode('utf-8') + os.linesep)
    except Exception as e:
        log.error(u'Невозможно сохранить информацию об ошибке в %s. %s' % (node.currentConverter.errorDest, e))

    try:
        if not os.path.exists(errorStatDir):
            os.makedirs(errorStatDir)
        fileName = errorStatDir + '/error_' + datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S') + '.txt'
        with open(fileName, 'w') as outfile:
            outfile.write(message.encode('utf-8'))
    except Exception as e:
        log.error(u'Невозможно сохранить информацию об ошибке в %s. %s' % (errorStatDir, e))


def createFileFlag(errorCount):
    """
    Создает файл-флаг, соответствующий настройкам в exchangers.ini,
    помещает внутрь количество ошибок
    """
    pathUploadFinishFlag = config.getString("UPLOAD", "pathUploadFinishFlag", None)

    if pathUploadFinishFlag:
        pathUploadFinishFlag = macros.applyMacros(pathUploadFinishFlag, log)
        log.info(u"Создаем файл-флаг: %s", pathUploadFinishFlag)
        directory = os.path.dirname(pathUploadFinishFlag)
        if not os.path.exists(directory):
            os.makedirs(directory)
        uploadFinishFlag = open(pathUploadFinishFlag, "w")
        if errorCount:
            uploadFinishFlag.write(str(errorCount))
        else:
            uploadFinishFlag.write("0")
        uploadFinishFlag.close()
